@@ -35,6 +35,7 @@
 
 typedef struct rp_window rp_window;
 typedef struct rp_screen rp_screen;
+typedef struct rp_global_screen rp_global_screen;
 typedef struct rp_action rp_action;
 typedef struct rp_keymap rp_keymap;
 typedef struct rp_frame rp_frame;
@@ -149,18 +150,37 @@ struct rp_group
   struct list_head node;
 };
 
+struct rp_global_screen
+{
+  Window root;
+  unsigned long fg_color, bg_color, fw_color, bw_color; /* The pixel color. */
+
+  /* This numset is responsible for giving out numbers for each screen */
+  struct numset *numset;
+};
+
+struct xrandr_info {
+  int output;
+  int crtc;
+  int primary;
+  struct sbuf* name;
+};
+
 struct rp_screen
 {
   GC normal_gc, inverse_gc;
   Window root, bar_window, key_window, input_window, frame_window, help_window;
   int bar_is_raised;
-  int screen_num;               /* Our screen number as dictated my X */
-  int xine_screen_num;          /* Our screen number for the Xinerama extension */
+  int screen_num;               /* Our screen number as dictated by X */
   Colormap def_cmap;
   Cursor rat;
-  unsigned long fg_color, bg_color, fw_color, bw_color; /* The pixel color. */
 
-  /* Here to abstract over the Xinerama vs X screens difference */
+  /* Screen number, handled by rp_global_screen numset */
+  int number;
+
+  struct xrandr_info xrandr;
+
+  /* Here to abstract over the Xrandr vs X screens difference */
   int left, top, width, height;
 
   char *display_string;
@@ -176,6 +196,12 @@ struct rp_screen
      when you switch screens the focus doesn't get frobbed. */
   int current_frame;
 
+  /* This structure can exist in a list. */
+  struct list_head node;
+
+  /* Used by sfrestore */
+  struct sbuf *scratch_buffer;
+
 #ifdef USE_XFT_FONT
   XftFont *xft_font;
   XftColor xft_fg_color, xft_bg_color;
@@ -332,6 +358,19 @@ struct modifier_info
 
 typedef struct list_head *(*completion_fn)(char *string);
 
+/*
+  BASIC: The completion shall begin with the same characters as the partial
+  string. Case is ignored.
+
+  SUBSTRING: The partial string shall be a subpart of the completion. Case
+  is ignored.
+*/
+enum completion_styles
+{
+  BASIC,
+  SUBSTRING
+};
+
 struct rp_completions
 {
   /* A pointer to the partial string that is being completed. We need
@@ -352,6 +391,9 @@ struct rp_completions
   /* virgin = 1 means no completions have been attempted on the input
      string. */
   unsigned short int virgin;
+
+  /* The completion style used to perform string comparisons */
+  enum completion_styles style;
 };
 
 struct rp_input_line
@@ -374,14 +416,6 @@ struct rp_hook_db_entry
   struct list_head *hook;
 };
 
-typedef struct rp_frame_undo rp_frame_undo;
-struct rp_frame_undo
-{
-  char *frames;
-  rp_screen *screen;
-  struct list_head node;
-};
-
 typedef struct rp_xselection rp_xselection;
 struct rp_xselection
 {
