@@ -25,6 +25,10 @@
 
 #include "ratpoison.h"
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #ifdef HAVE_HISTORY
 #include "readline/history.h"
 #endif
