@@ -109,6 +109,13 @@ init_xkb (void)
 KeySym
 keycode_to_keysym(Display *dpy, KeyCode kc, int group, int level)
 {
+  /*
+   * XKeycodeToKeysym has been deprecated upstream, however we still use
+   * it since XKB may not be available at build time or and not
+   * functional at runtime. The problems in XKeycodeToKeysym don't seem
+   * to matter in the ratpoison case anyway.
+   * https://bugs.freedesktop.org/show_bug.cgi?id=5349
+   */
 #if defined (WANT_XKB) && defined (HAVE_X11_XKBLIB_H) && defined (HAVE_XKBKEYCODETOKEYSYM)
   if (use_xkb)
     return XkbKeycodeToKeysym (dpy, kc, group, level);
@@ -117,47 +124,6 @@ keycode_to_keysym(Display *dpy, KeyCode kc, int group, int level)
   return XKeycodeToKeysym (dpy, kc, level);
 }
 
-/* /\* The caller is responsible for freeing the keycodes. *\/ */
-/* KeyCode * */
-/* keysym_to_keycodes (KeySym sym, int *n_returned) */
-/* { */
-/*   int min_code, max_code; */
-/*   int syms_per_code; */
-/*   KeySym *syms; */
-/*   KeyCode *codes; */
-/*   int code, code_col; */
-
-/*   XDisplayKeycodes (dpy, &min_code, &max_code); */
-/*   syms = XGetKeyboardMapping (dpy, */
-/* 			      min_code, max_code - min_code + 1, */
-/* 			      &syms_per_code); */
-
-/*   *n_returned = 0; */
-/*   codes = xmalloc (sizeof(KeyCode) * n_returned); */
-/*   for (code = min_code; code < max_code; code++) */
-/*     for (code_col = 0; code_col < syms_per_code; code_col++) */
-/*       {       */
-/* 	int s = syms[((code - min_code) * syms_per_code) + code_col]; */
-
-/* 	if (sym == s) */
-/* 	  { */
-/* 	    n_returned++; */
-/* 	    codes = xrealloc (sizeof(KeyCode) * n_returned); */
-/* 	    codes[n_returned-1] = code; */
-/* 	  } */
-/*       }	 */
-      
-/*   XFree ((char *) syms); */
-
-/*   if (n_returned > 0) */
-/*     return codes; */
-/*   else  */
-/*     { */
-/*       xfree (codes) */
-/*       return NULL; */
-/*     } */
-/* } */
-
 /* Figure out what keysyms are attached to what modifiers */
 void
 update_modifier_map (void)
@@ -439,7 +405,7 @@ read_single_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int
   int nbytes;
 
   XGetInputFocus (dpy, &focus, &revert);
-  set_window_focus (current_screen()->key_window);
+  set_window_focus (rp_current_screen->key_window);
   nbytes = read_key (keysym, modifiers, keysym_name, len);
   set_window_focus (focus);
 
@@ -508,7 +474,7 @@ update_input_window (rp_screen *s, rp_input_line *line)
                   line->length);
 
   gcv.function = GXxor;
-  gcv.foreground = s->fg_color ^ s->bg_color;
+  gcv.foreground = rp_glob_screen.fg_color ^ rp_glob_screen.bg_color;
   lgc = XCreateGC (dpy, s->input_window, GCFunction | GCForeground, &gcv);
 
   /* Draw a cheap-o cursor - MkIII */
@@ -530,12 +496,12 @@ ring_bell (void)
   GC lgc;
   XGCValues gcv;
   XWindowAttributes attr;
-  rp_screen *s = current_screen ();
+  rp_screen *s = rp_current_screen;
 
   XGetWindowAttributes (dpy, s->input_window, &attr);
 
   gcv.function = GXxor;
-  gcv.foreground = s->fg_color ^ s->bg_color;
+  gcv.foreground = rp_glob_screen.fg_color ^ rp_glob_screen.bg_color;
   lgc = XCreateGC (dpy, s->input_window, GCFunction | GCForeground, &gcv);
 
   XFillRectangle (dpy, s->input_window, lgc, 0, 0, attr.width, attr.height);
@@ -554,16 +520,16 @@ ring_bell (void)
 char *
 get_input (char *prompt, int history_id, completion_fn fn)
 {
-  return get_more_input (prompt, "", history_id, fn);
+  return get_more_input (prompt, "", history_id, BASIC, fn);
 }
 
 char *
 get_more_input (char *prompt, char *preinput, int history_id,
-                completion_fn compl_fn)
+                enum completion_styles style, completion_fn compl_fn)
 {
   /* Emacs 21 uses a 513 byte string to store the keysym name. */
   char keysym_buf[513];
-  rp_screen *s = current_screen ();
+  rp_screen *s = rp_current_screen;
   KeySym ch;
   unsigned int modifier;
   rp_input_line *line;
@@ -575,7 +541,7 @@ get_more_input (char *prompt, char *preinput, int history_id,
   history_reset();
 
   /* Create our line structure */
-  line = input_line_new (prompt, preinput, history_id, compl_fn);
+  line = input_line_new (prompt, preinput, history_id, style, compl_fn);
 
   /* We don't want to draw overtop of the program bar. */
   hide_bar (s);
