@@ -412,7 +412,7 @@ give_window_focus (rp_window *win, rp_window *last_win)
   if (last_win != NULL && win != last_win)
     {
       save_mouse_position (last_win);
-      XSetWindowBorder (dpy, last_win->w, last_win->scr->bw_color);
+      XSetWindowBorder (dpy, last_win->w, rp_glob_screen.bw_color);
     }
 
   if (win == NULL) return;
@@ -432,10 +432,10 @@ give_window_focus (rp_window *win, rp_window *last_win)
   if (last_win != NULL) XUninstallColormap (dpy, last_win->colormap);
   XInstallColormap (dpy, win->colormap);
 
-  XSetWindowBorder (dpy, win->w, win->scr->fw_color);
+  XSetWindowBorder (dpy, win->w, rp_glob_screen.fw_color);
 
   /* Finally, give the window focus */
-  rp_current_screen = win->scr->xine_screen_num;
+  rp_current_screen = win->scr;
   set_rp_window_focus (win);
 
   XSync (dpy, False);
@@ -452,100 +452,68 @@ void set_active_window_force (rp_window *win)
   set_active_window_body(win, 1);
 }
 
-/* FIXME: This function is probably a mess. I can't remember a time
-   when I didn't think this. It probably needs to be fixed up. */
+static rp_frame *
+find_frame_non_dedicated(rp_screen *current_screen, rp_frame *current_frame)
+{
+  rp_frame *cur;
+  rp_screen *screen;
+
+  list_for_each_entry (screen, &rp_screens, node)
+    {
+      if (current_screen == screen)
+        continue;
+
+      list_for_each_entry (cur, &screen->frames, node)
+        {
+          if (!cur->dedicated)
+            return cur;
+        }
+    }
+
+  return NULL;
+}
+
 void
 set_active_window_body (rp_window *win, int force)
 {
   rp_window *last_win;
   rp_frame *frame = NULL, *last_frame = NULL;
 
-  if (win == NULL) return;
+  if (win == NULL)
+    return;
 
   PRINT_DEBUG (("intended_frame_number: %d\n", win->intended_frame_number));
 
-  /* With Xinerama, we can move a window over to the current screen; otherwise
-   * we have to switch to the screen that the window belongs to.
-   */
-  if (rp_have_xinerama)
+  /* use the intended frame if we can. */
+  if (win->intended_frame_number >= 0)
     {
-      /* use the intended frame if we can. */
-      if (win->intended_frame_number >= 0)
-        {
-          frame = screen_get_frame (current_screen(), win->intended_frame_number);
-          win->intended_frame_number = -1;
-          if (frame != current_frame())
-            last_frame = current_frame();
-        }
-
-      if (!frame)
-        frame = screen_get_frame (current_screen(), current_screen()->current_frame);
+      frame = screen_get_frame (rp_current_screen, win->intended_frame_number);
+      win->intended_frame_number = -1;
+      if (frame != current_frame ())
+        last_frame = current_frame ();
     }
-  else
-    {
-      /* use the intended frame if we can. */
-      if (win->intended_frame_number >= 0)
-        {
-          frame = screen_get_frame (win->scr, win->intended_frame_number);
-          win->intended_frame_number = -1;
-          if (frame != current_frame())
-            last_frame = current_frame();
-        }
 
-      if (!frame)
-        frame = screen_get_frame (win->scr, win->scr->current_frame);
-    }
+  if (frame == NULL)
+    frame = screen_get_frame (rp_current_screen, rp_current_screen->current_frame);
 
   if (frame->dedicated && !force)
     {
-      /* Try to find a non-dedicated frame.  */
-      rp_frame *cur;
-      rp_screen *scr;
-      int done;
+      /* Try to find a non-dedicated frame. */
+      rp_frame *non_dedicated;
 
-      scr = (rp_have_xinerama)?&screens[rp_current_screen]:win->scr;
-      done = 0;
-
-      /* Try the only / current screen... */
-      for (cur = list_next_entry (frame, &scr->frames, node);
-           cur != frame && !done;
-           cur = list_next_entry (cur, &scr->frames, node))
+      non_dedicated = find_frame_non_dedicated (rp_current_screen, frame);
+      if (non_dedicated != NULL)
         {
-          if (!cur->dedicated)
-            {
-              set_active_frame (cur, 0);
-              last_frame = frame;
-              frame = cur;
-              done = 1;
-            }
-        }
-
-      /* If we have Xinerama, we can check *all* screens... */
-      if (rp_have_xinerama && !done)
-        {
-          int i;
-
-          for (i=0; i<num_screens && !done; i++)
-            {
-              if (scr == &screens[i]) continue;
-              list_for_each_entry (cur,&screens[i].frames,node)
-                {
-                  if (!cur->dedicated)
-                    {
-                      set_active_frame (cur, 0);
-                      last_frame = frame;
-                      frame = cur;
-                      done = 1; /* Break outer loop. */
-                      break;    /* Break inner loop. */
-                    }
-                }
-            }
+          last_frame = frame;
+          frame = non_dedicated;
+          set_active_frame (frame, 0);
         }
     }
 
   last_win = set_frames_window (frame, win);
 
-  if (last_win) PRINT_DEBUG (("last window: %s\n", window_name (last_win)));
+  if (last_win != NULL)
+    PRINT_DEBUG (("last window: %s\n", window_name (last_win)));
   PRINT_DEBUG (("new window: %s\n", window_name (win)));
 
   /* Make sure the window comes up full screen */
@@ -565,7 +533,7 @@ set_active_window_body (rp_window *win, int force)
   XSync (dpy, False);
 
   /* If we switched frame, go back to the old one. */
-  if (last_frame)
+  if (last_frame != NULL)
     set_active_frame (last_frame, 0);
 
   /* Call the switch window hook */
@@ -606,7 +574,7 @@ get_window_list (char *fmt, char *delim, struct sbuf *buffer,
   if (buffer == NULL) return;
 
   sbuf_clear (buffer);
-  find_window_other (current_screen());
+  find_window_other (rp_current_screen);
 
   /* We only loop through the current group to look for windows. */
   list_for_each_entry (we,&rp_current_group->mapped_windows,node)
@@ -683,3 +651,15 @@ win_get_frame (rp_window *win)
   else
     return NULL;
 }
+
+void
+change_windows_screen (rp_screen *old_screen, rp_screen *new_screen)
+{
+  rp_window *win;
+
+  list_for_each_entry (win, &rp_mapped_window, node)
+    {
+      if (win->scr == old_screen)
+        win->scr = new_screen;
+    }
+}
